ORDWAY_APP_DOMAIN = "app.ordwaylabs.com"
ORDWAY_APP_URL = f"https://{ORDWAY_APP_DOMAIN}"

ORDWAY_STAGING_DOMAIN = "staging.ordwaylabs.com"
ORDWAY_STAGING_URL = f"https://{ORDWAY_STAGING_DOMAIN}"

API_ENDPOINT_BASE = f"{ORDWAY_APP_URL}/api"
STAGING_ENDPOINT_BASE = f"{ORDWAY_STAGING_URL}/api"

SUPPORTED_API_VERSIONS = ("1",)
